@@ -5,6 +5,7 @@
 import atexit
 import multiprocessing
 import Queue
+import shlex
 import subprocess
 import sys
 import threading
@@ -24,12 +25,11 @@ class Group:
 		"""
 			Adds a new process to this object. This process is run and the output collected.
 			
-			@param cmd: the command to execute. If a string the whitespace tokenized before
-				passing to Popen
-			@return the handle to the process return from Popen
+			@param cmd: the command to execute. This may be an array as passed to Popen,
+				or a string, which will be parsed by 'shlex.split'
+			@return: the handle to the process return from Popen
 		"""
-		if isinstance(cmd, basestring):
-			cmd = filter(None,cmd.split( ' ' ))
+		cmd = _expand_cmd(cmd)
 			
 		handle = subprocess.Popen( cmd,
 			shell = shell,
@@ -171,3 +171,75 @@ class Group:
 			if not handle.group_output_done or handle.poll() == None:
 				nhandles.append( handle )
 		self.handles = nhandles
+
+
+class BadExitCode(Exception):
+	def __init__(self, exit_code, output):
+		Exception.__init__( self, 'subprocess-bad-exit-code' )
+		self.exit_code = exit_code
+		self.output = output
+	
+class Timeout(Exception):
+	def __init__(self, output):
+		Exception.__init__( self, 'subprocess-timeout' )
+		self.output = output
+		
+def call( cmd, encoding = 'utf-8', shell = False, check_exit_code = True, timeout = None ):
+	"""
+		Calls a subprocess and returns the output and optionally exit code.
+		
+		@param encoding: convert output to unicode objects with this encoding, set to None to
+			get the raw output
+		@param check_exit_code: set to False to ignore the exit code, otherwise any non-zero
+			result will throw BadExitCode.
+		@param timeout: If specified only this amount of time (seconds) will be waited for
+			the subprocess to return
+		@return: If check_exit_code is False: list( output, exit_code ), else just the output
+	"""
+	cmd = _expand_cmd(cmd)
+	proc = subprocess.Popen( cmd, stdout = subprocess.PIPE, stderr = subprocess.STDOUT,
+		stdin = subprocess.PIPE, shell = shell )
+
+	def decode(out):
+		if encoding != None:
+			return unicode( out, encoding )
+		else:
+			return raw_out
+	
+	if timeout == None:
+		raw_out, ignore_err = proc.communicate()
+	else:
+		# Read from subprocess in a thread so the main one can check for the timeout
+		outq = Queue.Queue()
+		def block_read():
+			# collect as lines so if timeout we still have partial output
+			out = proc.stdout.read()
+			outq.put( out )
+			
+		block_thread = threading.Thread( target = block_read )
+		block_thread.daemon = True
+		block_thread.start()
+
+		try:
+			raw_out = outq.get(True,timeout)
+		except Queue.Empty:
+			proc.terminate()
+			# wait again for partial output (process is terminated, so reading should end)
+			raw_out = outq.get()
+			raise Timeout( decode(raw_out) )
+	
+	out = decode(raw_out)
+	exit_code = proc.poll()
+	
+	if check_exit_code:
+		if exit_code != 0:
+			raise BadExitCode( exit_code, out )
+		return out
+		
+	return ( out, proc.poll() )
+	
+def _expand_cmd(cmd):
+	if isinstance(cmd, basestring):
+		cmd = shlex.split(cmd)
+	return cmd
+	
\ No newline at end of file
